# color.py
# ============================================================================== 
from termcolor import colored

def red(text):
    return colored(text, 'red')

def green(text):
    return colored(text, 'green')

def blue(text):
    return colored(text, 'blue')

def yellow(text):
    return colored(text, 'yellow')
